# -*- coding: utf-8 -*-

from django.conf import settings
import ldapdb.models
from ldapdb.models import fields as ldapdbfiels
import ldap

__all__ = ["LdapEcole", "LdapUser", "LdapClub", "LdapGroup", "LdapPosixGroup"]


class LdapEcole(ldapdb.models.Model):
    base_dn = settings.LDAP_BASE_DB
    object_classes = ["Ecole"]

    o = ldapdbfiels.CharField(db_column="o", unique=True, max_length=32)
    displayName = ldapdbfiels.CharField(db_column="displayName", max_length=50)

    def __str__(self):
        return self.o


class LdapUser(ldapdb.models.Model):
    base_dn = settings.LDAP_BASE_DB
    object_classes = ["Eleve"]

    login_inp_net = ldapdbfiels.CharField(db_column="uid", unique=True, max_length=200)
    mail = ldapdbfiels.CharField(db_column="mail", max_length=200)
    mail_redirect = ldapdbfiels.CharField(db_column="mailForwardingAddress", max_length=200, blank=True, null=True)
    bithday = ldapdbfiels.DateField(db_column="birthdate")
    nom = ldapdbfiels.CharField(db_column="sn", max_length=200)
    prenom = ldapdbfiels.CharField(db_column="givenName", max_length=200)
    ae = ldapdbfiels.CharField(db_column="inscritAE", max_length=50)
    promo = ldapdbfiels.IntegerField(db_column="promo")
    ecole = ldapdbfiels.CharField(db_column="ecole", max_length=50)

    def inscrit_AE(self):
        return self.ae == "TRUE"

    def get_ecole(self):
        return LdapEcole.objects.get(o=(self.ecole.split(",")[0][2:]))

    def __str__(self):
        return self.login_inp_net

    def check_password(self, password):
        con = ldap.initialize(settings.DATABASES["ldap"]["NAME"])
        try:
            con.bind_s(self.dn, password.encode("utf8"))
        except ldap.INVALID_CREDENTIALS:
            return False
        return True


class LdapClub(ldapdb.models.Model):
    base_dn = settings.LDAP_BASE_DB
    object_classes = ["Club"]

    cn = ldapdbfiels.CharField(db_column="cn", unique=True, max_length=128)
    displayName = ldapdbfiels.CharField(db_column="displayName", max_length=128)
    ecole = ldapdbfiels.CharField(db_column="ecole", max_length=50)
    membres = ldapdbfiels.ListField(db_column="memberUid")
    president = ldapdbfiels.CharField(db_column="president")
    vicesPresidents = ldapdbfiels.ListField(db_column="vicePresident")
    tresoriers = ldapdbfiels.ListField(db_column="tresorier")
    secretaire = ldapdbfiels.ListField(db_column="secretaire")

    def membres_bureau(self):
        return list(set(self.secretaire + self.tresoriers + self.vicesPresidents + [self.president]))

    def get_ecole(self):
        return LdapEcole.objects.get(o=(self.ecole.split(",")[0][2:]))

    def __str__(self):
        return self.cn


class LdapGroup(ldapdb.models.Model):
    base_dn = settings.LDAP_BASE_DB
    object_classes = ["Groupe"]

    cn = ldapdbfiels.CharField(db_column="cn", unique=True, max_length=128)
    displayName = ldapdbfiels.CharField(db_column="displayName", max_length=128)
    membres = ldapdbfiels.ListField(db_column="memberUid")

    def __str__(self):
        return self.cn


class LdapPosixGroup(ldapdb.models.Model):
    base_dn = settings.LDAP_BASE_DB
    object_classes = ["posixGroup"]

    cn = ldapdbfiels.CharField(db_column="cn", unique=True, max_length=128)
    membres = ldapdbfiels.ListField(db_column="memberUid")

    def __str__(self):
        return self.cn
